@@ -57,8 +57,6 @@ typedef struct
   Component *component;
   TurnServer *turn;
   StunAgent stun_agent;
-  uint8_t *msn_turn_username;
-  uint8_t *msn_turn_password;
   StunTimer timer;
   uint8_t stun_buffer[STUN_MAX_MESSAGE_SIZE];
   StunMessage stun_message;
@@ -70,16 +68,13 @@ typedef struct
 {
   NiceAgent *agent;         /**< back pointer to owner */
   NiceSocket *nicesock;     /**< existing socket to use */
-  NiceSocket *relay_socket; /**< relay socket from which we receive */
   NiceAddress server;       /**< STUN/TURN server address */
+  NiceCandidate *candidate; /**< candidate to refresh */
   Stream *stream;
   Component *component;
-  TurnServer *turn;
   StunAgent stun_agent;
   GSource *timer_source;
   GSource *tick_source;
-  uint8_t *msn_turn_username;
-  uint8_t *msn_turn_password;
   StunTimer timer;
   uint8_t stun_buffer[STUN_MAX_MESSAGE_SIZE];
   StunMessage stun_message;
@@ -87,13 +82,12 @@ typedef struct
   StunMessage stun_resp_msg;
 } CandidateRefresh;
 
-void refresh_free_item (gpointer data, gpointer user_data);
 void refresh_free (NiceAgent *agent);
 void refresh_prune_stream (NiceAgent *agent, guint stream_id);
+void refresh_prune_candidate (NiceAgent *agent, NiceCandidate *candidate);
 void refresh_cancel (CandidateRefresh *refresh);
 
 
-void discovery_free_item (gpointer data, gpointer user_data);
 void discovery_free (NiceAgent *agent);
 void discovery_prune_stream (NiceAgent *agent, guint stream_id);
 void discovery_schedule (NiceAgent *agent);
