@@ -27,7 +27,7 @@
 #endif
 
 #include "interfaces.h"
-#include "debug.h"
+#include "agent-priv.h"
 
 #ifdef G_OS_UNIX
 
@@ -51,7 +51,6 @@
 #endif
 
 #include <net/if.h>
-#include <net/if_arp.h>
 #include <arpa/inet.h>
 
 #ifdef HAVE_GETIFADDRS
@@ -143,25 +142,30 @@ nice_interfaces_get_local_interfaces (void)
 
 
 static gboolean
-nice_interfaces_is_private_ip (const struct sockaddr *sa)
+nice_interfaces_is_private_ip (const struct sockaddr *_sa)
 {
-  if (sa->sa_family == AF_INET) {
-    struct sockaddr_in *sa4 = (struct sockaddr_in *) sa;
+  union {
+    const struct sockaddr *addr;
+    const struct sockaddr_in *in;
+  } sa;
 
+  sa.addr = _sa;
+
+  if (sa.addr->sa_family == AF_INET) {
     /* 10.x.x.x/8 */
-    if (sa4->sin_addr.s_addr >> 24 == 0x0A)
+    if (sa.in->sin_addr.s_addr >> 24 == 0x0A)
       return TRUE;
 
     /* 172.16.0.0 - 172.31.255.255 = 172.16.0.0/10 */
-    if (sa4->sin_addr.s_addr >> 20 == 0xAC1)
+    if (sa.in->sin_addr.s_addr >> 20 == 0xAC1)
       return TRUE;
 
     /* 192.168.x.x/16 */
-    if (sa4->sin_addr.s_addr >> 16 == 0xC0A8)
+    if (sa.in->sin_addr.s_addr >> 16 == 0xC0A8)
       return TRUE;
 
     /* 169.254.x.x/16  (for APIPA) */
-    if (sa4->sin_addr.s_addr >> 16 == 0xA9FE)
+    if (sa.in->sin_addr.s_addr >> 16 == 0xA9FE)
       return TRUE;
   }
   
@@ -185,6 +189,14 @@ nice_interfaces_get_local_ips (gboolean include_loopback)
   for (ifa = results; ifa; ifa = ifa->ifa_next) {
     char addr_as_string[INET6_ADDRSTRLEN+1];
 
+    union {
+      struct sockaddr *addr;
+      struct sockaddr_in *in;
+      struct sockaddr_in6 *in6;
+    } sa;
+
+    sa.addr = ifa->ifa_addr;
+
     /* no ip address from interface that is down */
     if ((ifa->ifa_flags & IFF_UP) == 0)
       continue;
@@ -192,19 +204,15 @@ nice_interfaces_get_local_ips (gboolean include_loopback)
     if (ifa->ifa_addr == NULL) {
       continue;
     } else if (ifa->ifa_addr->sa_family == AF_INET) {
-      struct sockaddr_in *sa4 = (struct sockaddr_in *) ifa->ifa_addr;
-
-      if (inet_ntop (AF_INET, &sa4->sin_addr, addr_as_string,
+      if (inet_ntop (AF_INET, &sa.in->sin_addr, addr_as_string,
               INET6_ADDRSTRLEN) == NULL)
         continue;
     } else if (ifa->ifa_addr->sa_family == AF_INET6) {
-      struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)ifa->ifa_addr;
-
       /* Skip link-local addresses, they require a scope */
-      if (IN6_IS_ADDR_LINKLOCAL (&sa6->sin6_addr))
+      if (IN6_IS_ADDR_LINKLOCAL (&sa.in6->sin6_addr))
         continue;
 
-      if (inet_ntop (AF_INET6, &sa6->sin6_addr, addr_as_string,
+      if (inet_ntop (AF_INET6, &sa.in6->sin6_addr, addr_as_string,
               INET6_ADDRSTRLEN) == NULL)
         continue;
     } else
@@ -318,7 +326,10 @@ gchar *
 nice_interfaces_get_ip_for_interface (gchar *interface_name)
 {
   struct ifreq ifr;
-  struct sockaddr_in *sa;
+  union {
+    struct sockaddr *addr;
+    struct sockaddr_in *in;
+  } sa;
   gint sockfd;
 
 
@@ -339,17 +350,31 @@ nice_interfaces_get_ip_for_interface (gchar *interface_name)
   }
 
   close (sockfd);
-  sa = (struct sockaddr_in *) &ifr.ifr_addr;
-  nice_debug ("Address for %s: %s", interface_name, inet_ntoa (sa->sin_addr));
-  return g_strdup (inet_ntoa (sa->sin_addr));
+  sa.addr = &ifr.ifr_addr;
+  nice_debug ("Address for %s: %s", interface_name, inet_ntoa (sa.in->sin_addr));
+  return g_strdup (inet_ntoa (sa.in->sin_addr));
 }
 
 #else /* G_OS_UNIX */
 #ifdef G_OS_WIN32
 
 #include <winsock2.h>
-#include <Iphlpapi.h>
+#include <iphlpapi.h>
 
+// Should be in Iphlpapi.h, but mingw doesn't seem to have these
+// Values copied directly from:
+// http://msdn.microsoft.com/en-us/library/aa366845(v=vs.85).aspx
+// (Title: MIB_IPADDRROW structure)
+
+#ifndef MIB_IPADDR_DISCONNECTED
+#define MIB_IPADDR_DISCONNECTED 0x0008
+#endif
+
+#ifndef MIB_IPADDR_DELETED
+#define MIB_IPADDR_DELETED 0x0040
+#endif
+
+#if 0
 static gboolean started_wsa_engine = FALSE;
 
 /*
@@ -383,6 +408,7 @@ SOCKET nice_interfaces_get_WSA_socket ()
 
   return sock;
 }
+#endif
 
 GList * nice_interfaces_get_local_interfaces ()
 {
@@ -450,7 +476,7 @@ GList * nice_interfaces_get_local_ips (gboolean include_loopback)
             continue;
         }
 
-        ipstr = g_strdup_printf ("%d.%d.%d.%d",
+        ipstr = g_strdup_printf ("%lu.%lu.%lu.%lu",
             (ipaddr->dwAddr      ) & 0xFF,
             (ipaddr->dwAddr >>  8) & 0xFF,
             (ipaddr->dwAddr >> 16) & 0xFF,
@@ -471,8 +497,11 @@ GList * nice_interfaces_get_local_ips (gboolean include_loopback)
 /*
  * returns ip address as an utf8 string
  */
+// Source for idx's type (Was IF_INDEX):
+// http://msdn.microsoft.com/en-us/library/aa366836(v=VS.85).aspx
+// (Title: MIB_IFROW structure)
 static gchar *
-win32_get_ip_for_interface (IF_INDEX idx)
+win32_get_ip_for_interface (DWORD idx)
 {
   ULONG size = 0;
   PMIB_IPADDRTABLE ip_table;
@@ -491,7 +520,7 @@ win32_get_ip_for_interface (IF_INDEX idx)
       PMIB_IPADDRROW ipaddr = &ip_table->table[i];
       if (ipaddr->dwIndex == idx &&
           !(ipaddr->wType & (MIB_IPADDR_DISCONNECTED | MIB_IPADDR_DELETED))) {
-        ret = g_strdup_printf ("%d.%d.%d.%d",
+        ret = g_strdup_printf ("%lu.%lu.%lu.%lu",
             (ipaddr->dwAddr      ) & 0xFF,
             (ipaddr->dwAddr >>  8) & 0xFF,
             (ipaddr->dwAddr >> 16) & 0xFF,
