@@ -43,6 +43,7 @@
 #endif
 
 #include "pseudossl.h"
+#include "agent-priv.h"
 
 #include <string.h>
 
@@ -58,8 +59,8 @@ typedef struct {
 
 
 struct to_be_sent {
-  guint length;
-  gchar *buf;
+  guint8 *buf;  /* owned */
+  gsize length;
   NiceAddress to;
 };
 
@@ -89,19 +90,19 @@ static const gchar SSL_CLIENT_HANDSHAKE[] = {
 
 
 static void socket_close (NiceSocket *sock);
-static gint socket_recv (NiceSocket *sock, NiceAddress *from,
-    guint len, gchar *buf);
-static gboolean socket_send (NiceSocket *sock, const NiceAddress *to,
-    guint len, const gchar *buf);
+static gint socket_recv_messages (NiceSocket *sock,
+    NiceInputMessage *recv_messages, guint n_recv_messages);
+static gint socket_send_messages (NiceSocket *sock, const NiceAddress *to,
+    const NiceOutputMessage *messages, guint n_messages);
 static gboolean socket_is_reliable (NiceSocket *sock);
 
 static void add_to_be_sent (NiceSocket *sock, const NiceAddress *to,
-    const gchar *buf, guint len);
+    const NiceOutputMessage *messages, guint n_messages);
 static void free_to_be_sent (struct to_be_sent *tbs);
 
 
 NiceSocket *
-nice_pseudossl_socket_new (NiceAgent *agent, NiceSocket *base_socket)
+nice_pseudossl_socket_new (NiceSocket *base_socket)
 {
   PseudoSSLPriv *priv;
   NiceSocket *sock = g_slice_new0 (NiceSocket);
@@ -112,8 +113,8 @@ nice_pseudossl_socket_new (NiceAgent *agent, NiceSocket *base_socket)
 
   sock->fileno = priv->base_socket->fileno;
   sock->addr = priv->base_socket->addr;
-  sock->send = socket_send;
-  sock->recv = socket_recv;
+  sock->send_messages = socket_send_messages;
+  sock->recv_messages = socket_recv_messages;
   sock->is_reliable = socket_is_reliable;
   sock->close = socket_close;
 
@@ -142,28 +143,39 @@ socket_close (NiceSocket *sock)
 
 
 static gint
-socket_recv (NiceSocket *sock, NiceAddress *from, guint len, gchar *buf)
+socket_recv_messages (NiceSocket *sock,
+    NiceInputMessage *recv_messages, guint n_recv_messages)
 {
   PseudoSSLPriv *priv = sock->priv;
 
   if (priv->handshaken) {
-    if (priv->base_socket)
-      return nice_socket_recv (priv->base_socket, from, len, buf);
+    if (priv->base_socket) {
+      /* Fast path: once we’ve done the handshake, pass straight through to the
+       * base socket. */
+      return nice_socket_recv_messages (priv->base_socket,
+          recv_messages, n_recv_messages);
+    }
   } else {
-    gchar data[sizeof(SSL_SERVER_HANDSHAKE)];
-    gint ret  = -1;
+    guint8 data[sizeof(SSL_SERVER_HANDSHAKE)];
+    gint ret = -1;
+    GInputVector local_recv_buf = { data, sizeof (data) };
+    NiceInputMessage local_recv_message = { &local_recv_buf, 1, NULL, 0 };
 
-    if (priv->base_socket)
-      ret = nice_socket_recv (priv->base_socket, from, sizeof(data), data);
+    if (priv->base_socket) {
+      ret = nice_socket_recv_messages (priv->base_socket,
+          &local_recv_message, 1);
+    }
 
     if (ret <= 0) {
       return ret;
-    } else if ((guint) ret == sizeof(SSL_SERVER_HANDSHAKE) &&
+    } else if (ret == 1 &&
+        local_recv_buf.size == sizeof (SSL_SERVER_HANDSHAKE) &&
         memcmp(SSL_SERVER_HANDSHAKE, data, sizeof(SSL_SERVER_HANDSHAKE)) == 0) {
       struct to_be_sent *tbs = NULL;
       priv->handshaken = TRUE;
       while ((tbs = g_queue_pop_head (&priv->send_queue))) {
-        nice_socket_send (priv->base_socket, &tbs->to, tbs->length, tbs->buf);
+        nice_socket_send (priv->base_socket, &tbs->to, tbs->length,
+            (const gchar *) tbs->buf);
         g_free (tbs->buf);
         g_slice_free (struct to_be_sent, tbs);
       }
@@ -178,19 +190,22 @@ socket_recv (NiceSocket *sock, NiceAddress *from, guint len, gchar *buf)
   return 0;
 }
 
-static gboolean
-socket_send (NiceSocket *sock, const NiceAddress *to,
-    guint len, const gchar *buf)
+static gint
+socket_send_messages (NiceSocket *sock, const NiceAddress *to,
+    const NiceOutputMessage *messages, guint n_messages)
 {
   PseudoSSLPriv *priv = sock->priv;
 
   if (priv->handshaken) {
-    if (priv->base_socket)
-      return nice_socket_send (priv->base_socket, to, len, buf);
-    else
+    /* Fast path: pass directly through to the base socket once the handshake is
+     * complete. */
+    if (priv->base_socket == NULL)
       return FALSE;
+
+    return nice_socket_send_messages (priv->base_socket, to, messages,
+        n_messages);
   } else {
-    add_to_be_sent (sock, to, buf, len);
+    add_to_be_sent (sock, to, messages, n_messages);
   }
   return TRUE;
 }
@@ -205,21 +220,43 @@ socket_is_reliable (NiceSocket *sock)
 
 static void
 add_to_be_sent (NiceSocket *sock, const NiceAddress *to,
-    const gchar *buf, guint len)
+    const NiceOutputMessage *messages, guint n_messages)
 {
   PseudoSSLPriv *priv = sock->priv;
-  struct to_be_sent *tbs = NULL;
+  guint i;
 
-  if (len <= 0)
-    return;
+  for (i = 0; i < n_messages; i++) {
+    struct to_be_sent *tbs;
+    const NiceOutputMessage *message = &messages[i];
+    guint j;
+    gsize offset = 0;
+    gsize message_len;
 
-  tbs = g_slice_new0 (struct to_be_sent);
-  tbs->buf = g_memdup (buf, len);
-  tbs->length = len;
-  if (to)
-    tbs->to = *to;
-  g_queue_push_tail (&priv->send_queue, tbs);
+    tbs = g_slice_new0 (struct to_be_sent);
 
+    message_len = output_message_get_size (message);
+
+   /* Compact the buffer. */
+    tbs->buf = g_malloc (message_len);
+    tbs->length = message_len;
+    if (to != NULL)
+      tbs->to = *to;
+    g_queue_push_tail (&priv->send_queue, tbs);
+
+    for (j = 0;
+         (message->n_buffers >= 0 && j < (guint) message->n_buffers) ||
+         (message->n_buffers < 0 && message->buffers[j].buffer != NULL);
+         j++) {
+      const GOutputVector *buffer = &message->buffers[j];
+      gsize len;
+
+      len = MIN (message_len - offset, buffer->size);
+      memcpy (tbs->buf + offset, buffer->buffer, len);
+      offset += len;
+    }
+
+    g_assert (offset == message_len);
+  }
 }
 
 static void
