@@ -37,6 +37,7 @@
 #ifndef _SOCKET_H
 #define _SOCKET_H
 
+#include "agent.h"
 #include "address.h"
 #include <gio/gio.h>
 
@@ -58,10 +59,13 @@ struct _NiceSocket
 {
   NiceAddress addr;
   GSocket *fileno;
-  gint (*recv) (NiceSocket *sock, NiceAddress *from, guint len,
-      gchar *buf);
-  gboolean (*send) (NiceSocket *sock, const NiceAddress *to, guint len,
-      const gchar *buf);
+  /* Implementations must handle any value of n_recv_messages, including 0. Iff
+   * n_recv_messages is 0, recv_messages may be NULL. */
+  gint (*recv_messages) (NiceSocket *sock,
+      NiceInputMessage *recv_messages, guint n_recv_messages);
+  /* As above, @n_messages may be zero. Iff so, @messages may be %NULL. */
+  gint (*send_messages) (NiceSocket *sock, const NiceAddress *to,
+      const NiceOutputMessage *messages, guint n_messages);
   gboolean (*is_reliable) (NiceSocket *sock);
   void (*close) (NiceSocket *sock);
   void *priv;
@@ -69,11 +73,15 @@ struct _NiceSocket
 
 G_GNUC_WARN_UNUSED_RESULT
 gint
-nice_socket_recv (NiceSocket *sock, NiceAddress *from, guint len, gchar *buf);
+nice_socket_recv_messages (NiceSocket *sock,
+    NiceInputMessage *recv_messages, guint n_recv_messages);
 
-gboolean
-nice_socket_send (NiceSocket *sock, const NiceAddress *to,
-  guint len, const gchar *buf);
+gint
+nice_socket_send_messages (NiceSocket *sock, const NiceAddress *addr,
+    const NiceOutputMessage *messages, guint n_messages);
+gssize
+nice_socket_send (NiceSocket *sock, const NiceAddress *addr, gsize len,
+    const gchar *buf);
 
 gboolean
 nice_socket_is_reliable (NiceSocket *sock);
