@@ -57,10 +57,10 @@
 
 
 static void socket_close (NiceSocket *sock);
-static gint socket_recv (NiceSocket *sock, NiceAddress *from,
-    guint len, gchar *buf);
-static gboolean socket_send (NiceSocket *sock, const NiceAddress *to,
-    guint len, const gchar *buf);
+static gint socket_recv_messages (NiceSocket *sock,
+    NiceInputMessage *recv_messages, guint n_recv_messages);
+static gint socket_send_messages (NiceSocket *sock, const NiceAddress *to,
+    const NiceOutputMessage *messages, guint n_messages);
 static gboolean socket_is_reliable (NiceSocket *sock);
 
 struct UdpBsdSocketPrivate
@@ -72,7 +72,10 @@ struct UdpBsdSocketPrivate
 NiceSocket *
 nice_udp_bsd_socket_new (NiceAddress *addr)
 {
-  struct sockaddr_storage name;
+  union {
+    struct sockaddr_storage storage;
+    struct sockaddr addr;
+  } name;
   NiceSocket *sock = g_slice_new0 (NiceSocket);
   GSocket *gsock = NULL;
   gboolean gret = FALSE;
@@ -80,25 +83,25 @@ nice_udp_bsd_socket_new (NiceAddress *addr)
   struct UdpBsdSocketPrivate *priv;
 
   if (addr != NULL) {
-    nice_address_copy_to_sockaddr(addr, (struct sockaddr *)&name);
+    nice_address_copy_to_sockaddr(addr, &name.addr);
   } else {
     memset (&name, 0, sizeof (name));
-    name.ss_family = AF_UNSPEC;
+    name.storage.ss_family = AF_UNSPEC;
   }
 
-  if (name.ss_family == AF_UNSPEC || name.ss_family == AF_INET) {
+  if (name.storage.ss_family == AF_UNSPEC || name.storage.ss_family == AF_INET) {
     gsock = g_socket_new (G_SOCKET_FAMILY_IPV4, G_SOCKET_TYPE_DATAGRAM,
         G_SOCKET_PROTOCOL_UDP, NULL);
-    name.ss_family = AF_INET;
+    name.storage.ss_family = AF_INET;
 #ifdef HAVE_SA_LEN
-    name.ss_len = sizeof (struct sockaddr_in);
+    name.storage.ss_len = sizeof (struct sockaddr_in);
 #endif
-  } else if (name.ss_family == AF_INET6) {
+  } else if (name.storage.ss_family == AF_INET6) {
     gsock = g_socket_new (G_SOCKET_FAMILY_IPV6, G_SOCKET_TYPE_DATAGRAM,
         G_SOCKET_PROTOCOL_UDP, NULL);
-    name.ss_family = AF_INET6;
+    name.storage.ss_family = AF_INET6;
 #ifdef HAVE_SA_LEN
-    name.ss_len = sizeof (struct sockaddr_in6);
+    name.storage.ss_len = sizeof (struct sockaddr_in6);
 #endif
   }
 
@@ -109,7 +112,7 @@ nice_udp_bsd_socket_new (NiceAddress *addr)
 
   /* GSocket: All socket file descriptors are set to be close-on-exec. */
   g_socket_set_blocking (gsock, false);
-  gaddr = g_socket_address_new_from_native (&name, sizeof (name));
+  gaddr = g_socket_address_new_from_native (&name.addr, sizeof (name));
   if (gaddr != NULL) {
     gret = g_socket_bind (gsock, gaddr, FALSE, NULL);
     g_object_unref (gaddr);
@@ -124,7 +127,7 @@ nice_udp_bsd_socket_new (NiceAddress *addr)
 
   gaddr = g_socket_get_local_address (gsock, NULL);
   if (gaddr == NULL ||
-      !g_socket_address_to_native (gaddr, &name, sizeof(name), NULL)) {
+      !g_socket_address_to_native (gaddr, &name.addr, sizeof(name), NULL)) {
     g_slice_free (NiceSocket, sock);
     g_socket_close (gsock, NULL);
     g_object_unref (gsock);
@@ -133,14 +136,14 @@ nice_udp_bsd_socket_new (NiceAddress *addr)
 
   g_object_unref (gaddr);
 
-  nice_address_set_from_sockaddr (&sock->addr, (struct sockaddr *)&name);
+  nice_address_set_from_sockaddr (&sock->addr, &name.addr);
 
   priv = sock->priv = g_slice_new0 (struct UdpBsdSocketPrivate);
   nice_address_init (&priv->niceaddr);
 
   sock->fileno = gsock;
-  sock->send = socket_send;
-  sock->recv = socket_recv;
+  sock->send_messages = socket_send_messages;
+  sock->recv_messages = socket_recv_messages;
   sock->is_reliable = socket_is_reliable;
   sock->close = socket_close;
 
@@ -155,6 +158,7 @@ socket_close (NiceSocket *sock)
   if (priv->gaddr)
     g_object_unref (priv->gaddr);
   g_slice_free (struct UdpBsdSocketPrivate, sock->priv);
+  sock->priv = NULL;
 
   if (sock->fileno) {
     g_socket_close (sock->fileno, NULL);
@@ -164,60 +168,126 @@ socket_close (NiceSocket *sock)
 }
 
 static gint
-socket_recv (NiceSocket *sock, NiceAddress *from, guint len, gchar *buf)
+socket_recv_messages (NiceSocket *sock,
+    NiceInputMessage *recv_messages, guint n_recv_messages)
 {
-  GSocketAddress *gaddr = NULL;
-  GError *gerr = NULL;
-  gint recvd;
+  guint i;
+  gboolean error = FALSE;
 
-  recvd = g_socket_receive_from (sock->fileno, &gaddr, buf, len, NULL, &gerr);
+  /* Read messages into recv_messages until one fails or would block, or we
+   * reach the end. */
+  for (i = 0; i < n_recv_messages; i++) {
+    NiceInputMessage *recv_message = &recv_messages[i];
+    GSocketAddress *gaddr = NULL;
+    GError *gerr = NULL;
+    gssize recvd;
+    gint flags = G_SOCKET_MSG_NONE;
 
-  if (recvd < 0) {
-    if (g_error_matches(gerr, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)
-        || g_error_matches(gerr, G_IO_ERROR, G_IO_ERROR_FAILED))
-      recvd = 0;
+    recvd = g_socket_receive_message (sock->fileno,
+        (recv_message->from != NULL) ? &gaddr : NULL,
+        recv_message->buffers, recv_message->n_buffers, NULL, NULL,
+        &flags, NULL, &gerr);
 
-    g_error_free (gerr);
+    recv_message->length = MAX (recvd, 0);
+
+    if (recvd < 0) {
+      if (g_error_matches (gerr, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK))
+        recvd = 0;
+      else
+        error = TRUE;
+
+      g_error_free (gerr);
+    }
+
+    if (recvd > 0 && recv_message->from != NULL && gaddr != NULL) {
+      union {
+        struct sockaddr_storage storage;
+        struct sockaddr addr;
+      } sa;
+
+      g_socket_address_to_native (gaddr, &sa.addr, sizeof (sa), NULL);
+      nice_address_set_from_sockaddr (recv_message->from, &sa.addr);
+    }
+
+    if (gaddr != NULL)
+      g_object_unref (gaddr);
+
+    /* Return early on error or EWOULDBLOCK. */
+    if (recvd <= 0)
+      break;
   }
 
-  if (recvd > 0 && from != NULL && gaddr != NULL) {
-    struct sockaddr_storage sa;
+  /* Was there an error processing the first message? */
+  if (error && i == 0)
+    return -1;
 
-    g_socket_address_to_native (gaddr, &sa, sizeof (sa), NULL);
-    nice_address_set_from_sockaddr (from, (struct sockaddr *)&sa);
-  }
-
-  if (gaddr != NULL)
-    g_object_unref (gaddr);
-
-  return recvd;
+  return i;
 }
 
-static gboolean
-socket_send (NiceSocket *sock, const NiceAddress *to,
-    guint len, const gchar *buf)
+static gssize
+socket_send_message (NiceSocket *sock, const NiceAddress *to,
+    const NiceOutputMessage *message)
 {
   struct UdpBsdSocketPrivate *priv = sock->priv;
-  ssize_t sent;
+  GError *child_error = NULL;
+  gssize len;
 
   if (!nice_address_is_valid (&priv->niceaddr) ||
       !nice_address_equal (&priv->niceaddr, to)) {
-    struct sockaddr_storage sa;
+    union {
+      struct sockaddr_storage storage;
+      struct sockaddr addr;
+    } sa;
     GSocketAddress *gaddr;
 
     if (priv->gaddr)
       g_object_unref (priv->gaddr);
-    nice_address_copy_to_sockaddr (to, (struct sockaddr *)&sa);
-    gaddr = g_socket_address_new_from_native (&sa, sizeof(sa));
+
+    nice_address_copy_to_sockaddr (to, &sa.addr);
+    gaddr = g_socket_address_new_from_native (&sa.addr, sizeof(sa));
+    priv->gaddr = gaddr;
+
     if (gaddr == NULL)
       return -1;
-    priv->gaddr = gaddr;
+
     priv->niceaddr = *to;
   }
 
-  sent = g_socket_send_to (sock->fileno, priv->gaddr, buf, len, NULL, NULL);
+  len = g_socket_send_message (sock->fileno, priv->gaddr, message->buffers,
+      message->n_buffers, NULL, 0, G_SOCKET_MSG_NONE, NULL, &child_error);
 
-  return sent == (ssize_t)len;
+  if (len < 0) {
+    if (g_error_matches (child_error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK))
+      len = 0;
+
+    g_error_free (child_error);
+  }
+
+  return len;
+}
+
+static gint
+socket_send_messages (NiceSocket *sock, const NiceAddress *to,
+    const NiceOutputMessage *messages, guint n_messages)
+{
+  guint i;
+
+  for (i = 0; i < n_messages; i++) {
+    const NiceOutputMessage *message = &messages[i];
+    gssize len;
+
+    len = socket_send_message (sock, to, message);
+
+    if (len < 0) {
+      /* Error. */
+      return len;
+    } else if (len == 0) {
+      /* EWOULDBLOCK. */
+      break;
+    }
+  }
+
+  return i;
 }
 
 static gboolean
