@@ -645,7 +645,7 @@ StunMessageReturn stun_message_find_string (const StunMessage *msg,
  * %STUN_MESSAGE_RETURN_UNSUPPORTED_ADDRESS if the address family is unknown.
  */
 StunMessageReturn stun_message_find_addr (const StunMessage *msg,
-    StunAttribute type, struct sockaddr *addr, socklen_t *addrlen);
+    StunAttribute type, struct sockaddr_storage *addr, socklen_t *addrlen);
 
 /**
  * stun_message_find_xor_addr:
@@ -664,7 +664,7 @@ StunMessageReturn stun_message_find_addr (const StunMessage *msg,
  * %STUN_MESSAGE_RETURN_UNSUPPORTED_ADDRESS if the address family is unknown.
  */
 StunMessageReturn stun_message_find_xor_addr (const StunMessage *msg,
-    StunAttribute type, struct sockaddr *addr, socklen_t *addrlen);
+    StunAttribute type, struct sockaddr_storage *addr, socklen_t *addrlen);
 
 /**
  * stun_message_find_xor_addr_full:
@@ -684,7 +684,7 @@ StunMessageReturn stun_message_find_xor_addr (const StunMessage *msg,
  * %STUN_MESSAGE_RETURN_UNSUPPORTED_ADDRESS if the address family is unknown.
  */
 StunMessageReturn stun_message_find_xor_addr_full (const StunMessage *msg,
-    StunAttribute type, struct sockaddr *addr,
+    StunAttribute type, struct sockaddr_storage *addr,
     socklen_t *addrlen, uint32_t magic_cookie);
 
 
@@ -811,7 +811,7 @@ StunMessageReturn stun_message_append_addr (StunMessage * msg,
  * %STUN_MESSAGE_RETURN_UNSUPPORTED_ADDRESS if the address family is unknown.
  */
 StunMessageReturn stun_message_append_xor_addr (StunMessage * msg,
-    StunAttribute type, const struct sockaddr *addr, socklen_t addrlen);
+    StunAttribute type, const struct sockaddr_storage *addr, socklen_t addrlen);
 
 /**
  * stun_message_append_xor_addr_full:
@@ -828,7 +828,7 @@ StunMessageReturn stun_message_append_xor_addr (StunMessage * msg,
  * %STUN_MESSAGE_RETURN_UNSUPPORTED_ADDRESS if the address family is unknown.
  */
 StunMessageReturn stun_message_append_xor_addr_full (StunMessage * msg,
-    StunAttribute type, const struct sockaddr *addr, socklen_t addrlen,
+    StunAttribute type, const struct sockaddr_storage *addr, socklen_t addrlen,
     uint32_t magic_cookie);
 
 /**
