@@ -285,18 +285,18 @@ static void test_message (void)
 }
 
 
-bool test_attribute_validater (StunAgent *agent,
+static bool test_attribute_validater (StunAgent *agent,
     StunMessage *message, uint8_t *username, uint16_t username_len,
     uint8_t **password, size_t *password_len, void *user_data)
 {
-  char *pwd = (char *) user_data;
+  uint8_t *pwd = user_data;
 
   if (username_len != 4 ||
       memcmp (username, "ABCD", 4) != 0)
     return false;
 
   *password = pwd;
-  *password_len = strlen (pwd);
+  *password_len = strlen ((char *) pwd);
 
   return true;
 }
@@ -356,7 +356,7 @@ static void test_attribute (void)
 
   union
   {
-    struct sockaddr sa;
+    struct sockaddr_storage st;
     struct sockaddr_in6 s6;
   } addr;
   socklen_t addrlen;
@@ -368,7 +368,7 @@ static void test_attribute (void)
   StunMessage msg;
   uint16_t known_attributes[] = {STUN_ATTRIBUTE_MESSAGE_INTEGRITY, STUN_ATTRIBUTE_USERNAME, 0};
 
-  printf ("Attribute test message length: %lu\n", sizeof (acme));
+  printf ("Attribute test message length: %zd\n", sizeof (acme));
 
   stun_agent_init (&agent, known_attributes,
       STUN_COMPATIBILITY_RFC5389, STUN_AGENT_USAGE_SHORT_TERM_CREDENTIALS);
@@ -378,11 +378,11 @@ static void test_attribute (void)
     fatal ("Unauthorized validation failed");
 
   if (stun_agent_validate (&agent, &msg, acme, sizeof(acme),
-          test_attribute_validater, "bad__guy") != STUN_VALIDATION_UNAUTHORIZED)
+          test_attribute_validater, (void *) "bad__guy") != STUN_VALIDATION_UNAUTHORIZED)
     fatal ("invalid password validation failed");
 
   if (stun_agent_validate (&agent, &msg, acme, sizeof(acme),
-          test_attribute_validater, "good_guy") != STUN_VALIDATION_SUCCESS)
+          test_attribute_validater, (void *) "good_guy") != STUN_VALIDATION_SUCCESS)
     fatal ("good password validation failed");
 
   if (stun_message_has_attribute (&msg, 0xff00))
@@ -422,27 +422,27 @@ static void test_attribute (void)
     fatal ("String test failed");
 
   addrlen = sizeof (addr);
-  if (stun_message_find_addr (&msg, 0xff01, &addr.sa, &addrlen) !=
+  if (stun_message_find_addr (&msg, 0xff01, &addr.st, &addrlen) !=
       STUN_MESSAGE_RETURN_INVALID)
     fatal ("Too short addres test failed");
   addrlen = sizeof (addr);
-  if (stun_message_find_addr (&msg, 0xff02, &addr.sa, &addrlen) !=
+  if (stun_message_find_addr (&msg, 0xff02, &addr.st, &addrlen) !=
       STUN_MESSAGE_RETURN_UNSUPPORTED_ADDRESS)
     fatal ("Unknown address family test failed");
   addrlen = sizeof (addr);
-  if (stun_message_find_addr (&msg, 0xff03, &addr.sa, &addrlen) !=
+  if (stun_message_find_addr (&msg, 0xff03, &addr.st, &addrlen) !=
       STUN_MESSAGE_RETURN_INVALID)
     fatal ("Too short IPv6 address test failed");
   addrlen = sizeof (addr);
-  if (stun_message_find_addr (&msg, 0xff04, &addr.sa, &addrlen) !=
+  if (stun_message_find_addr (&msg, 0xff04, &addr.st, &addrlen) !=
       STUN_MESSAGE_RETURN_SUCCESS)
     fatal ("IPv4 address test failed");
   addrlen = sizeof (addr);
-  if (stun_message_find_addr (&msg, 0xff05, &addr.sa, &addrlen) !=
+  if (stun_message_find_addr (&msg, 0xff05, &addr.st, &addrlen) !=
       STUN_MESSAGE_RETURN_INVALID)
     fatal ("Too big IPv4 address test failed");
   addrlen = sizeof (addr);
-  if (stun_message_find_xor_addr (&msg, 0xff06, &addr.sa, &addrlen) !=
+  if (stun_message_find_xor_addr (&msg, 0xff06, &addr.st, &addrlen) !=
       STUN_MESSAGE_RETURN_SUCCESS ||
       memcmp (&addr.s6.sin6_addr, "\x20\x01\x0d\xb8""\xde\xad\xbe\xef"
                                   "\xde\xfa\xce\xd0""\xfa\xce\xde\xed", 16))
@@ -451,9 +451,9 @@ static void test_attribute (void)
 }
 
 static const char vector_username[] = "evtj:h6vY";
-static const char vector_password[] = "VOkJxbRl1RmTxUk/WvJxBt";
+static uint8_t vector_password[] = "VOkJxbRl1RmTxUk/WvJxBt";
 
-bool test_vector_validater (StunAgent *agent,
+static bool test_vector_validater (StunAgent *agent,
     StunMessage *message, uint8_t *username, uint16_t username_len,
     uint8_t **password, size_t *password_len, void *user_data)
 {
@@ -466,8 +466,8 @@ bool test_vector_validater (StunAgent *agent,
       memcmp (username, vector_username, strlen (vector_username)) != 0)
     fatal ("vector test : Validater received wrong username!");
 
-  *password = (uint8_t *) vector_password;
-  *password_len = strlen (vector_password);
+  *password = vector_password;
+  *password_len = strlen ((char *) vector_password);
 
 
   return true;
@@ -598,8 +598,11 @@ static void test_vectors (void)
 
        0x80, 0x28, 0x00, 0x04, // FINGERPRINT
        0xec, 0x27, 0xae, 0xb7};
-  struct sockaddr_in ip4;
-  struct sockaddr_in6 ip6;
+  union {
+    struct sockaddr_storage st;
+    struct sockaddr_in ip4;
+    struct sockaddr_in6 ip6;
+  } addr;
   socklen_t addrlen;
 
   StunAgent agent;
@@ -616,8 +619,7 @@ static void test_vectors (void)
       STUN_AGENT_USAGE_SHORT_TERM_CREDENTIALS |
       STUN_AGENT_USAGE_USE_FINGERPRINT);
 
-  memset (&ip4, 0, sizeof (ip4));
-  memset (&ip6, 0, sizeof (ip6));
+  memset (&addr, 0, sizeof (addr));
 
   puts ("Checking test vectors...");
 
@@ -635,7 +637,8 @@ static void test_vectors (void)
   if (stun_message_length (&msg) != sizeof(req) - 32)
     fatal ("vector test: removing attributes failed");
 
-  stun_agent_finish_message (&agent, &msg, vector_password, strlen (vector_password));
+  stun_agent_finish_message (&agent, &msg, vector_password,
+      strlen ((char *) vector_password));
 
   if (stun_message_length (&msg) != stun_message_length (&msg2) ||
       memcmp (req, req2, sizeof(req)) != 0)
@@ -649,15 +652,15 @@ static void test_vectors (void)
           test_vector_validater, (void *) 0) != STUN_VALIDATION_UNMATCHED_RESPONSE)
     fatal ("Response ipv4 test vector authentication failed");
 
-  addrlen = sizeof (ip4);
+  addrlen = sizeof (addr.ip4);
   if (stun_message_find_xor_addr (&msg, STUN_ATTRIBUTE_XOR_MAPPED_ADDRESS,
-          (struct sockaddr *)&ip4, &addrlen) != STUN_MESSAGE_RETURN_SUCCESS)
+          &addr.st, &addrlen) != STUN_MESSAGE_RETURN_SUCCESS)
     fatal ("Response test vector IPv4 extraction failed");
-  if (ip4.sin_family != AF_INET)
+  if (addr.ip4.sin_family != AF_INET)
     fatal ("Response test vector IPv4 family failed");
-  if (ntohl (ip4.sin_addr.s_addr) != 0xC0000201)
+  if (ntohl (addr.ip4.sin_addr.s_addr) != 0xC0000201)
     fatal ("Response test vector IPv4 address failed");
-  if (ntohs (ip4.sin_port) != 32853)
+  if (ntohs (addr.ip4.sin_port) != 32853)
     fatal ("Response test vector IPv6 port failed");
 
   if (stun_agent_validate (&agent, &msg, req, sizeof(req),
@@ -682,16 +685,16 @@ static void test_vectors (void)
           test_vector_validater, (void *) 1) != STUN_VALIDATION_SUCCESS)
     fatal ("Response ipv6 test vector authentication failed");
 
-  addrlen = sizeof (ip6);
+  addrlen = sizeof (addr.ip6);
   if (stun_message_find_xor_addr (&msg, STUN_ATTRIBUTE_XOR_MAPPED_ADDRESS,
-          (struct sockaddr *)&ip6, &addrlen) != STUN_MESSAGE_RETURN_SUCCESS)
+          &addr.st, &addrlen) != STUN_MESSAGE_RETURN_SUCCESS)
     fatal ("Response test vector IPv6 extraction failed");
-  if (ip6.sin6_family != AF_INET6)
+  if (addr.ip6.sin6_family != AF_INET6)
     fatal ("Response test vector IPv6 family failed");
-  if (memcmp (ip6.sin6_addr.s6_addr, "\x20\x01\x0d\xb8\x12\x34\x56\x78"
+  if (memcmp (addr.ip6.sin6_addr.s6_addr, "\x20\x01\x0d\xb8\x12\x34\x56\x78"
               "\x00\x11\x22\x33\x44\x55\x66\x77", 16) != 0)
     fatal ("Response test vector IPv6 address failed");
-  if (ntohs (ip6.sin6_port) != 32853)
+  if (ntohs (addr.ip6.sin6_port) != 32853)
     fatal ("Response test vector IPv6 port failed");
 
 
@@ -710,9 +713,9 @@ static void test_hash_creds (void)
   puts ("Testing long term credentials hash algorithm...");
 
 
-  stun_hash_creds ("realm", strlen ("realm"),
-      "user",  strlen ("user"),
-      "pass", strlen ("pass"), md5);
+  stun_hash_creds ((uint8_t *) "realm", strlen ("realm"),
+      (uint8_t *) "user",  strlen ("user"),
+      (uint8_t *) "pass", strlen ("pass"), md5);
 
   stun_debug ("key for user:realm:pass is : ");
   stun_debug_bytes (md5, 16);
