@@ -126,7 +126,7 @@ StunUsageBindReturn stun_usage_bind_process (StunMessage *msg,
       if ((code / 100) == 3) {
         if (alternate_server && alternate_server_len) {
           if (stun_message_find_addr (msg, STUN_ATTRIBUTE_ALTERNATE_SERVER,
-                  alternate_server,
+                  (struct sockaddr_storage *) alternate_server,
                   alternate_server_len) != STUN_MESSAGE_RETURN_SUCCESS) {
             stun_debug (" Unexpectedly missing ALTERNATE-SERVER attribute\n");
             return STUN_USAGE_BIND_RETURN_ERROR;
@@ -152,12 +152,14 @@ StunUsageBindReturn stun_usage_bind_process (StunMessage *msg,
   stun_debug ("Received %u-bytes STUN message\n", stun_message_length (msg));
 
   val = stun_message_find_xor_addr (msg,
-      STUN_ATTRIBUTE_XOR_MAPPED_ADDRESS, addr, addrlen);
+      STUN_ATTRIBUTE_XOR_MAPPED_ADDRESS, (struct sockaddr_storage *)addr,
+      addrlen);
   if (val != STUN_MESSAGE_RETURN_SUCCESS)
   {
     stun_debug (" No XOR-MAPPED-ADDRESS: %d\n", val);
     val = stun_message_find_addr (msg,
-        STUN_ATTRIBUTE_MAPPED_ADDRESS, addr, addrlen);
+        STUN_ATTRIBUTE_MAPPED_ADDRESS, (struct sockaddr_storage *)addr,
+        addrlen);
     if (val != STUN_MESSAGE_RETURN_SUCCESS)
     {
       stun_debug (" No MAPPED-ADDRESS: %d\n", val);
@@ -355,7 +357,7 @@ static int stun_err_dequeue (int fd)
 
 static ssize_t
 stun_trans_sendto (StunTransport *tr, const uint8_t *buf, size_t len,
-                     const struct sockaddr *dst, socklen_t dstlen)
+    const struct sockaddr *dst, socklen_t dstlen)
 {
   static const int flags = MSG_DONTWAIT | MSG_NOSIGNAL;
   ssize_t val;
@@ -375,14 +377,15 @@ stun_trans_sendto (StunTransport *tr, const uint8_t *buf, size_t len,
 
 static ssize_t
 stun_trans_recvfrom (StunTransport *tr, uint8_t *buf, size_t maxlen,
-                       struct sockaddr * dst,
+                       struct sockaddr_storage * dst,
                        socklen_t * dstlen)
 {
   static const int flags = MSG_DONTWAIT | MSG_NOSIGNAL;
   ssize_t val;
 
   if (dstlen != NULL)
-    val = recvfrom (tr->fd, (void *)buf, maxlen, flags, dst, dstlen);
+    val = recvfrom (tr->fd, (void *)buf, maxlen, flags, (struct sockaddr *) dst,
+        dstlen);
   else
     val = recv (tr->fd, (void *)buf, maxlen, flags);
 
@@ -396,14 +399,11 @@ stun_trans_recvfrom (StunTransport *tr, uint8_t *buf, size_t maxlen,
 static ssize_t
 stun_trans_send (StunTransport *tr, const uint8_t *buf, size_t len)
 {
-  union {
-    struct sockaddr_storage *storage;
-    struct sockaddr *addr;
-  } conv;
+  struct sockaddr *conv;
 
-  conv.storage = &tr->dst;
+  conv = (struct sockaddr *) &tr->dst;
 
-  return stun_trans_sendto (tr, buf, len, conv.addr, tr->dstlen);
+  return stun_trans_sendto (tr, buf, len, conv, tr->dstlen);
 }
 
 static ssize_t
@@ -454,7 +454,7 @@ stun_trans_poll (StunTransport *tr, unsigned int delay)
 
 /** Blocking mode STUN binding discovery */
 StunUsageBindReturn stun_usage_bind_run (const struct sockaddr *srv,
-    socklen_t srvlen, struct sockaddr *addr, socklen_t *addrlen)
+    socklen_t srvlen, struct sockaddr_storage *addr, socklen_t *addrlen)
 {
   StunTimer timer;
   StunTransport trans;
@@ -467,10 +467,7 @@ StunUsageBindReturn stun_usage_bind_run (const struct sockaddr *srv,
   size_t len;
   StunUsageTransReturn ret;
   int val;
-  union {
-    struct sockaddr_storage storage;
-    struct sockaddr addr;
-  } alternate_server;
+  struct sockaddr_storage alternate_server;
   socklen_t alternate_server_len = sizeof (alternate_server);
   StunUsageBindReturn bind_ret;
 
@@ -534,13 +531,13 @@ StunUsageBindReturn stun_usage_bind_run (const struct sockaddr *srv,
     if (valid != STUN_VALIDATION_SUCCESS) {
       ret = STUN_USAGE_TRANS_RETURN_RETRY;
     } else {
-      bind_ret = stun_usage_bind_process (&msg, addr, addrlen,
-          &alternate_server.addr, &alternate_server_len);
+      bind_ret = stun_usage_bind_process (&msg, (struct sockaddr *)  addr,
+          addrlen, (struct sockaddr *) &alternate_server, &alternate_server_len);
       if (bind_ret == STUN_USAGE_BIND_RETURN_ALTERNATE_SERVER) {
         stun_trans_deinit (&trans);
 
         ret = stun_trans_create (&trans, SOCK_DGRAM, 0,
-            &alternate_server.addr, alternate_server_len);
+            (struct sockaddr *) &alternate_server, alternate_server_len);
 
         if (ret != STUN_USAGE_TRANS_RETURN_SUCCESS) {
           return STUN_USAGE_BIND_RETURN_ERROR;
